use getset::{Getters, Setters};
use headless_chrome::Browser;
use strum::Display;

#[derive(Getters, Setters, Debug)]
#[getset(get = "pub")]
pub struct DiscordInvite {
    code: String,
}

impl DiscordInvite {
    pub fn new(code: String) -> Self {
        Self { code }
    }

    pub fn resolve(&self, browser: &Browser) -> Result<(), DiscordInviteError> {
        let mut tab = browser
            .new_tab()
            .map_err(|e| DiscordInviteError::Unknown(format!("Failed to create new tab: {}", e)))?;
        tab.navigate_to(&format!("https://discord.com/invite/{}", self.code))
            .map_err(|e| {
                DiscordInviteError::Unknown(format!("Failed to navigate to url: {}", e))
            })?;
        //std::thread::sleep(std::time::Duration::from_secs(4));
        tab.wait_until_navigated().map_err(|e| {
            DiscordInviteError::Unknown(format!("Failed to wait until navigated: {}", e))
        })?;
        let content = tab.get_content().unwrap();
        return match content.contains("Invite Invalid") {
            true => Err(DiscordInviteError::InvalidInvite),
            false => Ok(()),
        };
    }
}

impl From<String> for DiscordInvite {
    fn from(code: String) -> Self {
        Self::new(code)
    }
}

impl From<&str> for DiscordInvite {
    fn from(code: &str) -> Self {
        Self::new(code.to_string())
    }
}

#[derive(Debug, Display)]
pub enum DiscordInviteError {
    InvalidInvite,
    RateLimit,
    Timeout,
    Unknown(String),
}
